/**********************************************************************************************************************************/
/* Project      : TAPER                                                                                                           */
/* Module       : TPFILE.H                                                                                                        */
/* Description  : File handler                                                                                                    */
/* Version type : Module interface                                                                                                */
/* Last changed : 09-01-1999  19:00                                                                                               */
/* Update count : 8                                                                                                               */
/* OS type      : PC (DOS)                                                                                                        */
/* Compiler     : Watcom C32 Optimizing Compiler Version 10.x                                                                     */
/* Linker       : Watcom C Linker                                                                                                 */
/*                                                                                                                                */
/*                Copyleft (C) 1997-2001 ThunderWare Research Center, written by Martijn van der Heide.                           */
/*                                                                                                                                */
/*                This program is free software; you can redistribute it and/or                                                   */
/*                modify it under the terms of the GNU General Public License                                                     */
/*                as published by the Free Software Foundation; either version 2                                                  */
/*                of the License, or (at your option) any later version.                                                          */
/*                                                                                                                                */
/*                This program is distributed in the hope that it will be useful,                                                 */
/*                but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  */
/*                MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   */
/*                GNU General Public License for more details.                                                                    */
/*                                                                                                                                */
/*                You should have received a copy of the GNU General Public License                                               */
/*                along with this program; if not, write to the Free Software                                                     */
/*                Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                                     */
/*                                                                                                                                */
/**********************************************************************************************************************************/

#ifndef TPFILE_H_INC
#define TPFILE_H_INC

#include "tpconfig.h"

#ifdef EXTERN
#undef EXTERN
#endif
#ifdef __TPFILE_MAIN__
#define EXTERN
#else
#define EXTERN extern
#endif

/* Bad formats      */
#define FT_BADSOUND    -2                                                                       /* Wrong sound format .WAV or .AU */
#define FT_UNKNOWN     -1
/* Tape formats     */
#define FT_TAPZ80      0
#define FT_ZXS         1
#define FT_VOC         2
#define FT_TZX         3
#define FT_LTP         4
#define FT_ACORNRAW    5                                                                   /* Raw format of Z80Em (Acorn Risc PC) */
#define FT_CSW         6                                                                      /* Compressed Square Wave (Ramsoft) */
/* Snapshot formats */
#define FT_Z80         10                                                                                         /* Z80 snapshot */
#define FT_SNA         11                                                                                         /* SNA snapshot */

#define IsSnapshot(_Win)  (TapeWindow[_Win].FileType >= 10)

struct MicrodriveRecord_s                                                  /* One sector in a microdrive file     */
{
  byte                      HdFlag;                                        /* Value 1, to indicate header block   */
  byte                      HdNumb;                                        /* Sector number (values 254 to 1)     */
  word                      xxxxx1;                                        /* Unused bytes                        */
  char                      HdName[10];                                    /* Microdrive cartridge name (padded)  */
  byte                      HdChk;                                         /* Checksum of previous fields         */
  byte                      RecFlg;                                        /* Bit    0: Always 0 -indicate record */
                                                                           /* Bit    1: Set for the EOF block     */
                                                                           /* Bit    2: Reset for a PRINT file    */
                                                                           /* Bits 3-7: Unused bits (0)           */
  byte                      RecNum;                                        /* Datablock sequence number (first=0) */
  word                      RecLen;                                        /* Datablock length (<=512, LSB first) */
  char                      RecNam[10];                                    /* Filename (padded)                   */
  byte                      DesChk;                                        /* Record descriptor checksum          */
                                                                           /* Split datablock for headers:        */
  byte                        DType;                                       /* Type of datablock                   */
  word                        DLength;                                     /* Length of datablock                 */
  word                        DLoad;                                       /* Load address of datablock           */
  word                        DExtra;                                      /* Extra parameter of datablock        */
  word                        DAuto;                                       /* Autostart line if BASIC             */
  byte                        DBlock[503];                                 /* Rest of datablock                   */
  byte                      DChk;                                          /* Datablock checksum                  */
};

struct TapeRecord_s                                                        /* One header of a tape file           */
{
  byte                      HType;                                         /* Type of datablock                   */
  char                      HName[10];                                     /* Name of datablock                   */
  word                      HLength;                                       /* Length of following datablock       */
  word                      HStart;                                        /* Startaddress of following datablock */
  word                      HExtra;                                        /* Extra parameter of datablock        */
};

/**********************************************************************************************************************************/
/* Stuff needed for Spectrum token remapping                                                                                      */
/**********************************************************************************************************************************/

#define                     F_PAPER     0x0001
#define                     F_INK       0x0002
#define                     F_BRIGHT    0x0004
#define                     F_INVERSE   0x0008
#define                     F_FLASH     0x0010
#define                     F_OVER      0x0020
#define                     F_PAD_L     0x1000
#define                     F_PAD_R     0x2000
#define                     F_SKIP1     0x4000
#define                     F_SKIP2     0x8000

struct RemapCharacterSet_s
{
  char                     *Token;
  word                      Flags;
};

/**********************************************************************************************************************************/
/* Define the global variables                                                                                                    */
/**********************************************************************************************************************************/

EXTERN struct RemapCharacterSet_s    *RemapCharacterSet;
EXTERN bool                           FileNotFound;

/**********************************************************************************************************************************/
/* Define the interface functions                                                                                                 */
/**********************************************************************************************************************************/

void  RemapSpectrumUDGs    (bool Assume48K);

short RemapSpectrumString  (byte *SpectrumString, short Length, char *RemappedString, short MaxLength, bool DropColors,
                            bool InitialPad, short *RawLength, bool ToCombo, bool RealNumbers, bool DropTABs);

#endif
